@@ -5,6 +5,7 @@
 #include "qquick3dobject_p.h"
 #include "qquick3dtexture_p.h"
 #include "qquick3dcubemaptexture_p.h"
+#include "qquick3ddebugsettings_p.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -15,18 +16,124 @@ QT_BEGIN_NAMESPACE
     \brief Lets you configure how a scene is rendered.
 
     SceneEnvironment defines a set of global properties for how a scene should be rendered.
+
+    \note The QtQuick3D.Helpers module offers an \l ExtendedSceneEnvironment
+    type which inherits from SceneEnvironment and adds a number of built-in
+    effects on top.
+
+    To use SceneEnvironment or \l ExtendedSceneEnvironment, associate the
+    \l{View3D::environment}{environment property} of a View3D with an instance
+    of these types. The object can be declared inline, for example like this:
+
+    \qml
+        View3D {
+            environment: SceneEnvironment {
+                antialiasingMode: SceneEnvironment.MSAA
+                tonemapMode: SceneEnvironment.TonemapModeFilmic
+                backgroundMode: SceneEnvironment.SkyBox
+                lightProbe: Texture {
+                    source: "panoramic_hdri_background.hdr"
+                }
+            }
+        }
+    \endqml
+
+    Alternatively, the environment object can be defined separately. It can
+    then be referenced by one or more View3D objects. An example code snippet,
+    using \l ExtendedSceneEnvironment this time:
+
+    \qml
+        ExtendedSceneEnvironment {
+            id: myEnv
+            vignetteEnabled: true
+        }
+
+        View3D {
+            width: parent.width / 2
+            environment: myEnv
+        }
+
+        View3D {
+            width: parent.width / 2
+            x: parent.width / 2
+            environment: myEnv
+        }
+    \endqml
+
+    \section1 Feature Overview
+
+    \list
+
+    \li Anti-aliasing settings. See \l{Anti-Aliasing Best Practices} for an
+    overview of this topic. The relevant properties are \l antialiasingMode, \l
+    antialiasingQuality, \l specularAAEnabled, \l temporalAAEnabled, \l
+    temporalAAStrength. In addition, if \l ExtendedSceneEnvironment is used,
+    another method is available via
+    \l{ExtendedSceneEnvironment::fxaaEnabled}{fxaaEnabled}.
+
+    \li Screen space ambient occlusion. The relevant properties are
+    \l aoEnabled, \l aoStrength, \l aoBias, \l aoDistance, \l aoDither,
+    \l aoSampleRate, \l aoSoftness.
+
+    \li Clear color, skybox, image-based lighting. For more information on IBL,
+    see \l{Using Image-Based Lighting}. The relevant properties are \l
+    backgroundMode, \l clearColor, \l lightProbe, \l probeExposure, \l
+    probeHorizon, \l probeOrientation, \l skyboxBlurAmount, \l skyBoxCubeMap.
+
+    \li Tonemapping. \l tonemapMode configures the tonemapping method that is
+    used to convert the high dynamic range color values to the 0-1 range at the
+    end of the graphics pipeline. \l ExtendedSceneEnvironment offers a few
+    additional properties, such as
+    \l{ExtendedSceneEnvironment::whitePoint}{whitePoint} and
+    \l{ExtendedSceneEnvironment::sharpnessAmount}{sharpnessAmount} that can be
+    used to tune the tonemapping calculations.
+
+    \li Depth buffer settings. The relevant properties are \l
+    depthPrePassEnabled, \l depthTestEnabled.
+
+    \li Post-processing effects. In addition to the built-in post-processing
+    effects provided by \l ExtendedSceneEnvironment, applications can provide
+    their own custom effects via the \l Effect type. The \l effects property is
+    a list of \l Effect instances.
+
+    \li Debug visualization settings, such as wireframe mode or rendering only
+    certain color contributions for the materials. This is controlled by the \l
+    DebugSettings object referenced from the \l debugSettings property. Most of
+    these settings can also be controlled interactively when a \l DebugView
+    item is added to the scene.
+
+    \li Fog settings. To enable fog, set an appropriately configured
+    \l [QML] Fog object in the \l [QML] fog property.
+
+    \li Lightmap baking settings. When pre-baked lightmaps are used for some
+    models in the scene, the \l Lightmapper object set in the \l lightmapper
+    property defines the settings used during the baking process.
+
+    \li Scissor settings. To apply a scissor different than the viewport, set
+    the \l scissorRect property.
+
+    \endlist
+
+    \sa ExtendedSceneEnvironment
 */
 
 QQuick3DSceneEnvironment::QQuick3DSceneEnvironment(QQuick3DObject *parent)
     : QQuick3DObject(*(new QQuick3DObjectPrivate(QQuick3DObjectPrivate::Type::SceneEnvironment)), parent)
 {
-
+    m_debugSettings = new QQuick3DDebugSettings(this);
+    m_debugSettingsSignalConnection = QObject::connect(m_debugSettings, &QQuick3DDebugSettings::changed, this,
+                                                       [this] { update(); });
+    QObject::connect(m_debugSettings, &QObject::destroyed, this,
+                     [this](QObject *obj) {
+        if (m_debugSettings == obj) {
+            m_debugSettings = nullptr;
+            update();
+        }
+    });
 }
 
 QQuick3DSceneEnvironment::~QQuick3DSceneEnvironment()
 {
-    for (const auto& connection : m_connections)
-        disconnect(connection);
 }
 
 /*!
@@ -53,8 +160,12 @@ QQuick3DSceneEnvironment::~QQuick3DSceneEnvironment()
     silhouettes.
 
     \b Cons: Usually more expensive than MSAA. Increases video memory usage.
-    Supported with View3D items with all renderMode except Inline as the
-    technique implies rendering to a texture first.
+    Supported with View3D items with all renderMode except Inline, but since
+    the technique implies rendering to a texture first, enabling SSAA with a
+    renderMode of Underlay or Overlay will result in using an intermediate
+    texture and render pass that would normally not be needed, meaning the
+    performance costs may be more noticeable. It is recommended to use SSAA
+    only when the renderMode is the default Offscreen.
 
     \b Multisampling
 
@@ -72,7 +183,7 @@ QQuick3DSceneEnvironment::~QQuick3DSceneEnvironment()
     depending on the renderMode. When using Underlay/Overlay with an effect
     applied or Offscreen, MSAA can be controlled for each View3D item. On the
     other hand, using Underlay/Overlay without any effect or Inline will make
-    MSAA contolled per-window.
+    MSAA controlled per-window.
 
     \note For View3D items with a \l{QtQuick3D::View3D::renderMode}{renderMode}
     other than Underlay/Overlay with effects or Offscreen, multisampling can only
@@ -95,6 +206,12 @@ QQuick3DSceneEnvironment::~QQuick3DSceneEnvironment()
     \b Cons: Does not take effect if any visual changes are occurring.
     Expensive due to having to accumulate and blend. Increases video memory
     usage.
+
+    \note Progressing antialiasing is not currently supported with multiview
+    rendering, and should not be used in VR/AR applications.
+
+    See \l{Anti-Aliasing Best Practices} for further discussion on
+    anti-aliasing methods.
 */
 QQuick3DSceneEnvironment::QQuick3DEnvironmentAAModeValues QQuick3DSceneEnvironment::antialiasingMode() const
 {
@@ -137,15 +254,16 @@ QQuick3DSceneEnvironment::QQuick3DEnvironmentAAQualityValues QQuick3DSceneEnviro
     This property controls if and how the background of the scene should be
     cleared.
 
-    \note Clearing does not always happen: depending on the
-    \l{QtQuick3D::View3D::renderMode}{renderMode} property the View3D may not
-    perform any clearing on its own, in which case
-    SceneEnvironment.Transparent and SceneEnvironment.Color have no effect.
-    Only the Offscreen mode (rendering into a texture) supports all clearing
-    modes. With the Underlay mode, use \l{QQuickWindow::setColor()} or
-    \l[QtQuick]{Window::color}{Window.color} to control the clear color for
-    the Qt Quick scene. SkyBox is handled differently, as it implies drawing
-    actual geometry, so that works identically across all render modes.
+    \note The clearing of the color buffer backing the View 3D does not always
+    happen: depending on the \l{QtQuick3D::View3D::renderMode}{renderMode}
+    property the View3D may not perform any clearing on its own, in which case
+    \c{SceneEnvironment.Transparent} and \c{SceneEnvironment.Color} have no
+    effect. Only the default \c Offscreen \l{View3D::renderMode}{render mode}
+    (rendering into a texture) supports all clearing modes. With the \c
+    Underlay mode, use \l{QQuickWindow::setColor()} or
+    \l[QtQuick]{Window::color}{Window.color} to control the clear color for the
+    Qt Quick scene. SkyBox is handled differently, as it implies drawing actual
+    geometry, so that works identically across all render modes.
 
     \value SceneEnvironment.Transparent
         The scene is cleared to be transparent. This is useful to render 3D content on top of another item.
@@ -164,7 +282,80 @@ QQuick3DSceneEnvironment::QQuick3DEnvironmentAAQualityValues QQuick3DSceneEnviro
 
     The default value is \c SceneEnvironment.Transparent
 
-    \sa QQuickWindow::setColor(), Window::color
+    Take the following example. The Suzanne model is expected to be
+    pre-processed with the \c balsam tool and is sourced from the
+    \l{https://github.com/KhronosGroup/glTF-Sample-Models}{glTF Sample Models}
+    repository.
+
+    \qml
+        import QtQuick
+        import QtQuick3D
+        import QtQuick3D.Helpers
+
+        Item {
+            width: 1280
+            height: 720
+
+            View3D {
+                id: v3d
+                anchors.fill: parent
+
+                environment: ExtendedSceneEnvironment {
+                    backgroundMode: SceneEnvironment.SkyBox
+                    lightProbe: Texture { source: "00455_OpenfootageNET_field_low.hdr" }
+
+                    glowEnabled: true
+                    glowStrength: 1.25
+                    glowBloom: 0.25
+                    glowBlendMode: ExtendedSceneEnvironment.GlowBlendMode.Additive
+                }
+
+                DirectionalLight {
+                }
+
+                Suzanne {
+                    scale: Qt.vector3d(50, 50, 50)
+                    z: -500
+                }
+
+                PerspectiveCamera {
+                    id: camera
+                }
+
+                WasdController {
+                    controlledObject: camera
+                }
+            }
+        }
+    \endqml
+
+    Using image-based lighting in additional to the DirectionalLight and also
+    using the light probe texture as the skybox gives us the following:
+
+    \image sceneenvironment_background_ibl.jpg
+
+    What happens if there is no light probe?
+
+    \qml
+        backgroundMode: SceneEnvironment.Transparent
+    \endqml
+
+    Here the background is provided not by the View3D but by the QQuickWindow
+    or QQuickView hosting the 2D and 3D scene. Lighting is based on the
+    DirectionalLight only.
+
+    \image sceneenvironment_background_transparent.jpg
+
+    Using a fixed clear color:
+
+    \qml
+        backgroundMode: SceneEnvironment.Color
+        clearColor: "green"
+    \endqml
+
+    \image sceneenvironment_background_color.jpg
+
+    \sa lightProbe, QQuickWindow::setColor(), Window::color, View3D
 */
 
 QQuick3DSceneEnvironment::QQuick3DEnvironmentBackgroundTypes QQuick3DSceneEnvironment::backgroundMode() const
@@ -189,7 +380,7 @@ QColor QQuick3DSceneEnvironment::clearColor() const
 }
 
 /*!
-    \qmlproperty float QtQuick3D::SceneEnvironment::aoStrength
+    \qmlproperty real QtQuick3D::SceneEnvironment::aoStrength
 
     This property defines the amount of ambient occulusion applied. Ambient
     occulusion is a form of approximated global illumination which causes
@@ -202,6 +393,28 @@ QColor QQuick3DSceneEnvironment::clearColor() const
     All values other than 0 have the same impact to the performance.
 
     The default value is 0.0. The maximum value is 100.0.
+
+    A value of 0 is equivalent to setting \l aoEnabled to false.
+
+    Pictured here with the default aoSoftness and aoDistance:
+
+    \table
+    \header
+    \li aoStrength of 0 (AO disabled)
+    \li aoStrength of 100
+    \li aoStrength of 50
+    \row
+    \li \image sceneenvironment_ao_off.jpg
+    \li \image sceneenvironment_ao_full_strength.jpg
+    \li \image sceneenvironment_ao_half_strength.jpg
+    \endtable
+
+    \note Getting visually good-looking screen space ambient occlusion is
+    dependent on carefully tuning a number of related parameters, such as \l
+    aoStrength, \l aoSoftness, \l aoDistance, \l aoDither, \l aoBias, and \l
+    aoSampleRate.
+
+    \sa aoEnabled, aoDistance, aoSoftness
 */
 float QQuick3DSceneEnvironment::aoStrength() const
 {
@@ -209,12 +422,30 @@ float QQuick3DSceneEnvironment::aoStrength() const
 }
 
 /*!
-    \qmlproperty float QtQuick3D::SceneEnvironment::aoDistance
+    \qmlproperty real QtQuick3D::SceneEnvironment::aoDistance
 
     This property defines roughly how far ambient occlusion shadows spread away
     from objects. Greater distances cause increasing impact to performance.
 
     The default value is 5.0.
+
+    Pictured here with the default aoSoftness and the maximum aoStrength:
+
+    \table
+    \header
+    \li aoDistance of 5
+    \li aoDistance of 1
+    \row
+    \li \image sceneenvironment_ao_distance_5.jpg
+    \li \image sceneenvironment_ao_distance_1.jpg
+    \endtable
+
+    \note Getting visually good-looking screen space ambient occlusion is
+    dependent on carefully tuning a number of related parameters, such as \l
+    aoStrength, \l aoSoftness, \l aoDistance, \l aoDither, \l aoBias, and \l
+    aoSampleRate.
+
+    \sa aoStrength, aoSoftness
 */
 float QQuick3DSceneEnvironment::aoDistance() const
 {
@@ -222,11 +453,30 @@ float QQuick3DSceneEnvironment::aoDistance() const
 }
 
 /*!
-    \qmlproperty float QtQuick3D::SceneEnvironment::aoSoftness
+    \qmlproperty real QtQuick3D::SceneEnvironment::aoSoftness
 
-    This property how smooth the edges of the ambient occlusion shading are.
+    This property defines how smooth the edges of the ambient occlusion shading
+    are.
 
     The value must be between 0.0 and 50.0. The default value is 50.0.
+
+    Pictured here with the default aoDistance and the maximum aoStrength:
+
+    \table
+    \header
+    \li aoSoftness of 50
+    \li aoSoftness of 25
+    \row
+    \li \image sceneenvironment_ao_softness_default.jpg
+    \li \image sceneenvironment_ao_softness_half.jpg
+    \endtable
+
+    \note Getting visually good-looking screen space ambient occlusion is
+    dependent on carefully tuning a number of related parameters, such as \l
+    aoStrength, \l aoSoftness, \l aoDistance, \l aoDither, \l aoBias, and \l
+    aoSampleRate.
+
+    \sa aoStrength, aoDistance
 */
 float QQuick3DSceneEnvironment::aoSoftness() const
 {
@@ -269,7 +519,7 @@ int QQuick3DSceneEnvironment::aoSampleRate() const
 }
 
 /*!
-    \qmlproperty float QtQuick3D::SceneEnvironment::aoBias
+    \qmlproperty real QtQuick3D::SceneEnvironment::aoBias
 
     This property defines a cutoff distance preventing objects from exhibiting
     ambient occlusion at close distances. Higher values increase the distance
@@ -305,6 +555,153 @@ float QQuick3DSceneEnvironment::aoBias() const
     supported in combination with this property. Pre-filtering of all mip
     levels for dynamic Qt Quick content is typically not reasonable in practice
     due to performance implications.
+
+    For more information on image-based lighting, see \l{Using Image-Based Lighting}.
+
+    \note The light probe texture, when the property is set to a valid Texture,
+    is used for lighting regardless of the \l backgroundMode. However, when \l
+    backgroundMode is set to \c{SceneEnvironment.SkyBox}, the texture is also
+    used to render the scene background as a skybox.
+
+    The examples below were generated with varying the \l backgroundMode in the
+    environment of the following scene. The scene has no DirectionLight,
+    PointLight, or SpotLight. All lighting is based on the panoramic HDRI
+    image.
+
+    \qml
+        import QtQuick
+        import QtQuick3D
+        import QtQuick3D.Helpers
+
+        Item {
+            width: 1280
+            height: 720
+
+            View3D {
+                id: v3d
+                anchors.fill: parent
+
+                environment: ExtendedSceneEnvironment {
+                    backgroundMode: SceneEnvironment.SkyBox
+                    lightProbe: Texture { source: "00455_OpenfootageNET_field_low.hdr" }
+
+                    tonemapMode: SceneEnvironment.TonemapModeFilmic
+                    sharpnessAmount: 0.4
+
+                    glowEnabled: true
+                    glowStrength: 1.25
+                    glowBloom: 0.25
+                    glowBlendMode: ExtendedSceneEnvironment.GlowBlendMode.Additive
+                }
+
+                Node {
+                    scale: Qt.vector3d(100, 100, 100)
+
+                    Sponza {
+                    }
+
+                    Suzanne {
+                        y: 1
+                        scale: Qt.vector3d(0.5, 0.5, 0.5)
+                        eulerRotation.y: -90
+                    }
+                }
+
+                PerspectiveCamera {
+                    id: camera
+                    y: 100
+                }
+
+                WasdController {
+                    controlledObject: camera
+                }
+            }
+        }
+    \endqml
+
+    Results with the above environment:
+
+    \image sceneenvironment_lightprobe.jpg
+    \image sceneenvironment_lightprobe_2.jpg
+
+    Switching the backgroundMode to \c{SceneEnvironment.Transparent} would give us:
+
+    \image sceneenvironment_lightprobe_transparent.jpg
+    \image sceneenvironment_lightprobe_transparent_2.jpg
+
+    Here the lighting of the 3D scene is the same as before, meaning the
+    materials use the light probe in the lighting calculations the same way as
+    before, but there is no skybox rendered. The background is white since that
+    is the default clear color of the QQuickWindow hosting the 2D and 3D scene.
+
+    It is valid to set the lightProbe property value back to the default null.
+    This unassigns the previously associated texture. For example, let's use
+    the Delete key to dynamically toggle between image-based lighting with a
+    skybox, and no image-based lighting with a fixed clear color for the
+    background:
+
+    \qml
+        environment: ExtendedSceneEnvironment {
+            id: env
+
+            backgroundMode: SceneEnvironment.SkyBox
+            lightProbe: iblTex
+
+            tonemapMode: SceneEnvironment.TonemapModeFilmic
+            sharpnessAmount: 0.4
+
+            glowEnabled: true
+            glowStrength: 1.25
+            glowBloom: 0.25
+            glowBlendMode: ExtendedSceneEnvironment.GlowBlendMode.Additive
+        }
+
+        Texture {
+            id: iblTex
+            source: "00455_OpenfootageNET_field_low.hdr"
+        }
+
+        focus: true
+        Keys.onDeletePressed: {
+            if (env.backgroundMode == SceneEnvironment.SkyBox) {
+                env.backgroundMode = SceneEnvironment.Color;
+                env.clearColor = "green";
+                env.lightProbe = null;
+            } else {
+                env.backgroundMode = SceneEnvironment.SkyBox;
+                env.lightProbe = iblTex;
+            }
+        }
+    \endqml
+
+    Pressing Delete gives the following result. Remember that the scene used
+    here has no lights so all 3D models appear completely black.
+
+    \image sceneenvironment_lightprobe_null.jpg
+    \image sceneenvironment_lightprobe_null_2.jpg
+
+    While lightProbe is commonly used in combination with Texture instances
+    that source their data from an image file (typically .hdr or .ktx), it can
+    also makes sense to associate with a Texture that uses in-memory,
+    \l{Texture::textureData}{procedurally generated image data}. A prime
+    example of this is a Texture where the image data is generated by \l
+    ProceduralSkyTextureData from the QtQuick3D.Helpers module:
+
+    \qml
+            backgroundMode: SceneEnvironment.SkyBox
+            lightProbe: Texture {
+                textureData: ProceduralSkyTextureData {
+                }
+            }
+    \endqml
+
+    This gives us a procedurally generated HDR skybox texture that is now used
+    both as the skybox and for image-based lighting:
+
+    \image sceneenvironment_lightprobe_proceduralsky.jpg
+
+    \sa backgroundMode, {Using Image-Based Lighting}, {Pre-generating IBL
+    cubemap}, probeExposure, probeHorizon, probeOrientation, ProceduralSkyTextureData
 */
 QQuick3DTexture *QQuick3DSceneEnvironment::lightProbe() const
 {
@@ -312,7 +709,7 @@ QQuick3DTexture *QQuick3DSceneEnvironment::lightProbe() const
 }
 
 /*!
-    \qmlproperty float QtQuick3D::SceneEnvironment::probeExposure
+    \qmlproperty real QtQuick3D::SceneEnvironment::probeExposure
 
     This property modifies the amount of light emitted by the light probe. Part
     of the tonemapping is exposure mapping, and this property adjusts how
@@ -322,6 +719,8 @@ QQuick3DTexture *QQuick3DSceneEnvironment::lightProbe() const
 
     \note This property does not have an effect when \l tonemapMode is set to
     \c SceneEnvironment.TonemapModeNone.
+
+    \sa lightProbe, probeHorizon, probeOrientation
 */
 float QQuick3DSceneEnvironment::probeExposure() const
 {
@@ -329,7 +728,7 @@ float QQuick3DSceneEnvironment::probeExposure() const
 }
 
 /*!
-    \qmlproperty float QtQuick3D::SceneEnvironment::probeHorizon
+    \qmlproperty real QtQuick3D::SceneEnvironment::probeHorizon
 
     This property when defined with increasing values adds darkness (black)
     to the bottom half of the environment, forcing the lighting to come
@@ -349,6 +748,8 @@ float QQuick3DSceneEnvironment::probeExposure() const
 
     \note The probeHorizon property only affects materials lighting, and has
     no effect on the rendering of the sky box.
+
+    \sa lightProbe, probeExposure, probeOrientation
 */
 float QQuick3DSceneEnvironment::probeHorizon() const
 {
@@ -363,6 +764,8 @@ float QQuick3DSceneEnvironment::probeHorizon() const
 
     \note This value augments how the lightProbe Texture is sampled in combination
     with any texture rotations and offsets set on the lightProbe texture.
+
+    \sa lightProbe, probeHorizon, probeExposure
 */
 QVector3D QQuick3DSceneEnvironment::probeOrientation() const
 {
@@ -385,6 +788,11 @@ QVector3D QQuick3DSceneEnvironment::probeOrientation() const
     lost; low impact on performance.
 
     \b Cons: Fast-moving objects cause one-frame ghosting.
+
+    \default false
+
+    \note Temporal antialiasing is not currently supported with multiview
+    rendering, and should not be used in VR/AR applications.
 */
 bool QQuick3DSceneEnvironment::temporalAAEnabled() const
 {
@@ -392,12 +800,14 @@ bool QQuick3DSceneEnvironment::temporalAAEnabled() const
 }
 
 /*!
-    \qmlproperty float QtQuick3D::SceneEnvironment::temporalAAStrength
+    \qmlproperty real QtQuick3D::SceneEnvironment::temporalAAStrength
     \since 5.15
 
     This property modifies the amount of temporal movement (antialiasing).
     This has an effect only when temporalAAEnabled property is true.
 
+    \default 0.3
+
     \sa temporalAAEnabled
 */
 float QQuick3DSceneEnvironment::temporalAAStrength() const
@@ -409,7 +819,20 @@ float QQuick3DSceneEnvironment::temporalAAStrength() const
     \qmlproperty bool QtQuick3D::SceneEnvironment::specularAAEnabled
     \since 6.4
 
-    When this property is enabled specular aliasing will be mitigated.
+    When this property is enabled, specular aliasing will be mitigated.
+    Specular aliasing is often visible in form of bright dots, possibly
+    flickering when moving the camera around.
+
+    The default value is false.
+
+    \table
+    \header
+    \li Specular AA disabled
+    \li Specular AA enabled
+    \row
+    \li \image specular_aa_off.jpg
+    \li \image specular_aa_on.jpg
+    \endtable
 */
 bool QQuick3DSceneEnvironment::specularAAEnabled() const
 {
@@ -504,7 +927,7 @@ QQmlListProperty<QQuick3DEffect> QQuick3DSceneEnvironment::effects()
 }
 
 /*!
-    \qmlproperty enumeration QQuick3D::SceneEnvironment::tonemapMode
+    \qmlproperty enumeration QtQuick3D::SceneEnvironment::tonemapMode
     \since 6.0
 
     This property defines how colors are tonemapped before rendering. All
@@ -528,10 +951,21 @@ QQmlListProperty<QQuick3DEffect> QQuick3DSceneEnvironment::effects()
     \value SceneEnvironment.TonemapModeFilmic
         Filmic tonemapping is applied.
 
-    \note When using post processing effects, many effects expect untonemapped
-    linear color data. It is important to bypass the built-in tonemapping in
-    this case by using the \c SceneEnvironment.TonemapModeNone value.
+    See \l{ExtendedSceneEnvironment::tonemapMode}{ExtendedSceneEnvironment} for
+    an example of these different modes.
 
+    \note When using post-processing effects, most effects expect untonemapped
+    linear color data. With application-provided, custom effects implemented
+    via the \l Effect type, it is important to know that starting with Qt 6.5
+    effects can safely assume that they work with linear color data, and
+    tonemapping is performed automatically on the output of the last effect in
+    the chain. If there is a need to customize tonemapping completely, consider
+    setting the \c SceneEnvironment.TonemapModeNone value to disable the
+    built-in tonemapper, and perform the appropriate adjustments on the color
+    value in the last effect in the chain instead. This does not apply to the
+    built-in effects of \l ExtendedSceneEnvironment, because those
+    automatically take care of proper tonemapping regardless of what
+    combination of built-in effects are enabled in the environment.
 */
 QQuick3DSceneEnvironment::QQuick3DEnvironmentTonemapModes QQuick3DSceneEnvironment::tonemapMode() const
 {
@@ -539,13 +973,12 @@ QQuick3DSceneEnvironment::QQuick3DEnvironmentTonemapModes QQuick3DSceneEnvironme
 }
 
 /*!
-    \qmlproperty float QQuick3D::SceneEnvironment::skyboxBlurAmount
+    \qmlproperty real QtQuick3D::SceneEnvironment::skyboxBlurAmount
     \since 6.4
 
     This property determines how much much the skybox should be blurred when
-    using \c SceneEnvironment.SkyBox for the
-    \l{QtQuick3D::View3D::backgroundMode}{backgroundMode} property. The default
-    value is \c 0.0 which means there is no blurring.
+    using \c SceneEnvironment.SkyBox for the \l backgroundMode property. The
+    default value is \c 0.0 which means there is no blurring.
 
     Acceptable values range between 0.0 and 1.0, all other values will be clamped
     to this range.
@@ -557,6 +990,46 @@ float QQuick3DSceneEnvironment::skyboxBlurAmount() const
     return m_skyboxBlurAmount;
 }
 
+/*!
+    \qmlproperty QtQuick3D::DebugSettings QtQuick3D::SceneEnvironment::debugSettings
+    \since 6.5
+
+    This property specifies a \c DebugSettings object which is used to
+    configure the debugging tools of the renderer. During construction
+    the SceneEnvironment automatically creates a DebugSettings object
+    associated with itself, and therefore setting a custom DebugSettings
+    is usually not required.
+
+    An example of rendering the scene with wireframe mode enabled:
+    \image debugsettings_wireframe.jpg
+
+    Visualizing the normal vectors of the meshes:
+    \image debugsettings_normals.jpg
+
+    Visualizing the specular lighting contribution:
+    \image debugsettings_specular.jpg
+
+    \sa DebugSettings
+*/
+
+QQuick3DDebugSettings *QQuick3DSceneEnvironment::debugSettings() const
+{
+    return m_debugSettings;
+}
+
+/*!
+    \qmlproperty rect QtQuick3D::SceneEnvironment::scissorRect
+    \since 6.5
+
+    This property defines a scissor rectangle in view coordinates, with the
+    top-left corner at [0, 0]
+*/
+
+QRect QQuick3DSceneEnvironment::scissorRect() const
+{
+    return m_scissorRect;
+}
+
 void QQuick3DSceneEnvironment::setAntialiasingMode(QQuick3DSceneEnvironment::QQuick3DEnvironmentAAModeValues antialiasingMode)
 {
     if (m_antialiasingMode == antialiasingMode)
@@ -603,6 +1076,10 @@ void QQuick3DSceneEnvironment::setAoStrength(float aoStrength)
         return;
 
     m_aoStrength = aoStrength;
+
+    const bool aoEnabled = !(qFuzzyIsNull(m_aoStrength) || qFuzzyIsNull(m_aoDistance));
+    setAoEnabled(aoEnabled);
+
     emit aoStrengthChanged();
     update();
 }
@@ -613,6 +1090,10 @@ void QQuick3DSceneEnvironment::setAoDistance(float aoDistance)
         return;
 
     m_aoDistance = aoDistance;
+
+    const bool aoEnabled = !(qFuzzyIsNull(m_aoStrength) || qFuzzyIsNull(m_aoDistance));
+    setAoEnabled(aoEnabled);
+
     emit aoDistanceChanged();
     update();
 }
@@ -662,10 +1143,7 @@ void QQuick3DSceneEnvironment::setLightProbe(QQuick3DTexture *lightProbe)
     if (m_lightProbe == lightProbe)
         return;
 
-    QQuick3DObjectPrivate::updatePropertyListener(lightProbe, m_lightProbe, QQuick3DObjectPrivate::get(this)->sceneManager, QByteArrayLiteral("lightProbe"), m_connections,
-                           [this](QQuick3DObject *n) {
-        setLightProbe(qobject_cast<QQuick3DTexture *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DSceneEnvironment::setLightProbe, lightProbe, m_lightProbe);
 
     m_lightProbe = lightProbe;
     emit lightProbeChanged();
@@ -735,6 +1213,11 @@ void QQuick3DSceneEnvironment::setTonemapMode(QQuick3DSceneEnvironment::QQuick3D
     update();
 }
 
+bool QQuick3DSceneEnvironment::useBuiltinTonemapper() const
+{
+    return true;
+}
+
 QSSGRenderGraphObject *QQuick3DSceneEnvironment::updateSpatialNode(QSSGRenderGraphObject *node)
 {
     // Don't do anything, these properties get set by the scene renderer
@@ -747,6 +1230,11 @@ void QQuick3DSceneEnvironment::itemChange(QQuick3DObject::ItemChange change, con
         updateSceneManager(value.sceneManager);
 }
 
+const QVector<QQuick3DEffect *> &QQuick3DSceneEnvironment::effectList() const
+{
+    return m_effects;
+}
+
 void QQuick3DSceneEnvironment::updateSceneManager(QQuick3DSceneManager *manager)
 {
     if (manager) {
@@ -798,6 +1286,9 @@ void QQuick3DSceneEnvironment::qmlAppendEffect(QQmlListProperty<QQuick3DEffect>
     if (effect->parentItem() == nullptr)
         effect->setParentItem(self);
 
+    for (QQuick3DEffect *e : self->m_effects)
+        e->effectChainDirty();
+
     self->update();
 }
 
@@ -833,7 +1324,7 @@ void QQuick3DSceneEnvironment::setSkyboxBlurAmount(float newSkyboxBlurAmount)
 }
 
 /*!
-    \qmlproperty Lightmapper SceneEnvironment::lightmapper
+    \qmlproperty Lightmapper QtQuick3D::SceneEnvironment::lightmapper
 
     When this property is set to a valid Lightmapper object, the settings
     specified by the object will be taken into account when baking lightmaps.
@@ -870,10 +1361,12 @@ void QQuick3DSceneEnvironment::setLightmapper(QQuick3DLightmapper *lightmapper)
                                                      [this] { update(); });
 
     QObject::connect(m_lightmapper, &QObject::destroyed, this,
-                     [this]
+                     [this](QObject *obj)
     {
-        m_lightmapper = nullptr;
-        update();
+        if (m_lightmapper == obj) {
+            m_lightmapper = nullptr;
+            update();
+        }
     });
 
     emit lightmapperChanged();
@@ -897,14 +1390,171 @@ void QQuick3DSceneEnvironment::setSkyBoxCubeMap(QQuick3DCubeMapTexture *newSkyBo
     if (m_skyBoxCubeMap == newSkyBoxCubeMap)
         return;
 
-
-    QQuick3DObjectPrivate::updatePropertyListener(newSkyBoxCubeMap, m_skyBoxCubeMap, QQuick3DObjectPrivate::get(this)->sceneManager, QByteArrayLiteral("skyboxCubeMap"), m_connections,
-                           [this](QQuick3DObject *n) {
-        setSkyBoxCubeMap(qobject_cast<QQuick3DCubeMapTexture *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DSceneEnvironment::setSkyBoxCubeMap, newSkyBoxCubeMap, m_skyBoxCubeMap);
 
     m_skyBoxCubeMap = newSkyBoxCubeMap;
     emit skyBoxCubeMapChanged();
 }
 
+void QQuick3DSceneEnvironment::setDebugSettings(QQuick3DDebugSettings *newDebugSettings)
+{
+    if (m_debugSettings == newDebugSettings)
+        return;
+
+    if (m_debugSettings)
+        m_debugSettings->disconnect(m_debugSettingsSignalConnection);
+
+    m_debugSettings = newDebugSettings;
+
+    m_debugSettingsSignalConnection = QObject::connect(m_debugSettings, &QQuick3DDebugSettings::changed, this,
+                                                       [this] { update(); });
+    QObject::connect(m_debugSettings, &QObject::destroyed, this,
+                     [this](QObject *obj) {
+        if (m_debugSettings == obj) {
+            m_debugSettings = nullptr;
+            update();
+        }
+    });
+
+    emit debugSettingsChanged();
+    update();
+}
+
+void QQuick3DSceneEnvironment::setScissorRect(QRect rect)
+{
+    if (m_scissorRect == rect)
+        return;
+
+    m_scissorRect = rect;
+    emit scissorRectChanged();
+    update();
+}
+
+bool QQuick3DSceneEnvironment::gridEnabled() const
+{
+    return m_gridEnabled;
+}
+
+void QQuick3DSceneEnvironment::setGridEnabled(bool newGridEnabled)
+{
+    if (m_gridEnabled == newGridEnabled)
+        return;
+    m_gridEnabled = newGridEnabled;
+    update();
+}
+
+float QQuick3DSceneEnvironment::gridScale() const
+{
+    return m_gridScale;
+}
+
+void QQuick3DSceneEnvironment::setGridScale(float newGridScale)
+{
+    if (qFuzzyCompare(m_gridScale, newGridScale))
+        return;
+    m_gridScale = newGridScale;
+    update();
+}
+
+uint QQuick3DSceneEnvironment::gridFlags() const
+{
+    return m_gridFlags;
+}
+
+void QQuick3DSceneEnvironment::setGridFlags(uint newGridFlags)
+{
+    if (m_gridFlags == newGridFlags)
+        return;
+    m_gridFlags = newGridFlags;
+    update();
+}
+
+/*!
+    \qmlproperty bool SceneEnvironment::aoEnabled
+    \since 6.5
+
+    Enable or disable ambient occlusion.
+
+    The default value is \c false, which means ambient occlusion is disabled.
+
+    \note If \l aoStrength or \ aoDistance is 0, then setting this property to
+    \c true will also set those values appropriately to make the ambient
+    occlusion effective.
+
+    \note Getting visually good-looking screen space ambient occlusion is
+    dependent on carefully tuning a number of related parameters, such as \l
+    aoStrength, \l aoSoftness, \l aoDistance, \l aoDither, \l aoBias, and \l
+    aoSampleRate.
+
+    \sa aoStrength, aoDistance
+*/
+
+bool QQuick3DSceneEnvironment::aoEnabled() const
+{
+    return m_aoEnabled;
+}
+
+void QQuick3DSceneEnvironment::setAoEnabled(bool newAoEnabled)
+{
+    if (m_aoEnabled == newAoEnabled)
+        return;
+
+    m_aoEnabled = newAoEnabled;
+
+    if (m_aoEnabled) {
+        if (qFuzzyIsNull(m_aoStrength))
+            setAoStrength(100.0f);
+        if (qFuzzyIsNull(m_aoDistance))
+            setAoDistance(defaultAoDistance());
+    }
+
+    emit aoEnabledChanged();
+    update();
+}
+
+/*!
+    \qmlproperty QtQuick3D::Fog QtQuick3D::SceneEnvironment::fog
+    \since 6.5
+
+    When this property is set to a valid \l {QtQuick3D::Fog}{Fog} object, it is
+    used to configure the renderer's built-in fog support.
+
+    The default value is null, which means no fog. This is equivalent to
+    setting a Fog object with \l{Fog::enabled}{enabled} set to false.
+
+    \image fog.jpg
+
+    \sa {QtQuick3D::Fog}{Fog}
+ */
+
+QQuick3DFog *QQuick3DSceneEnvironment::fog() const
+{
+    return m_fog;
+}
+
+void QQuick3DSceneEnvironment::setFog(QQuick3DFog *fog)
+{
+    if (m_fog == fog)
+        return;
+
+    if (m_fog)
+        m_fog->disconnect(m_fogSignalConnection);
+
+    m_fog = fog;
+
+    m_fogSignalConnection = QObject::connect(m_fog, &QQuick3DFog::changed, this, [this] { update(); });
+
+    QObject::connect(m_fog, &QObject::destroyed, this,
+                     [this](QObject *obj)
+    {
+        if (m_fog == obj) {
+            m_fog = nullptr;
+            update();
+        }
+    });
+
+    emit fogChanged();
+    update();
+}
+
 QT_END_NAMESPACE
