@@ -8,11 +8,16 @@
 #include <QtQml/QQmlFile>
 #include <QtQuick/QQuickItem>
 #include <QtQuick/private/qquickitem_p.h>
+#include <QSGTextureProvider>
 #include <QtCore/qmath.h>
 
+#include <ssg/qssgrenderextensions.h>
+
 #include "qquick3dobject_p.h"
 #include "qquick3dscenemanager_p.h"
 #include "qquick3dutils_p.h"
+#include "qquick3drenderextensions.h"
+#include "qquick3dviewport_p.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -84,7 +89,7 @@ QT_BEGIN_NAMESPACE
     \li \image spheremap.png
     \endtable
 
-    \sa {Qt Quick 3D - Dynamic Texture Example}, {Qt Quick 3D - Procedural Texture Example}
+    \sa {Qt Quick 3D - Procedural Texture Example}
 */
 
 QQuick3DTexture::QQuick3DTexture(QQuick3DObject *parent)
@@ -115,9 +120,6 @@ QQuick3DTexture::~QQuick3DTexture()
         QQuickItemPrivate *sourcePrivate = QQuickItemPrivate::get(m_sourceItem);
         sourcePrivate->removeItemChangeListener(this, QQuickItemPrivate::Geometry);
     }
-
-    for (const auto &connection : std::as_const(m_connections))
-        disconnect(connection);
 }
 
 /*!
@@ -211,7 +213,7 @@ QQuickItem *QQuick3DTexture::sourceItem() const
 }
 
 /*!
-    \qmlproperty float QtQuick3D::Texture::scaleU
+    \qmlproperty real QtQuick3D::Texture::scaleU
 
     This property defines how to scale the U texture coordinate when mapping to
     a mesh's UV coordinates.
@@ -236,7 +238,7 @@ float QQuick3DTexture::scaleU() const
 }
 
 /*!
-    \qmlproperty float QtQuick3D::Texture::scaleV
+    \qmlproperty real QtQuick3D::Texture::scaleV
 
     This property defines how to scale the V texture coordinate when mapping to
     a mesh's UV coordinates.
@@ -325,7 +327,24 @@ QQuick3DTexture::TilingMode QQuick3DTexture::verticalTiling() const
 }
 
 /*!
-    \qmlproperty float QtQuick3D::Texture::rotationUV
+    \qmlproperty enumeration QtQuick3D::Texture::tilingModeDepth
+
+    This property controls how the texture is mapped when the Z scaling value
+    is greater than 1.
+
+    By default, this property is set to \c{Texture.Repeat}.
+
+    \value Texture.ClampToEdge Texture is not tiled, but the value on the edge is used instead.
+    \value Texture.MirroredRepeat Texture is repeated and mirrored over the Z axis.
+    \value Texture.Repeat Texture is repeated over the Z axis.
+*/
+QQuick3DTexture::TilingMode QQuick3DTexture::depthTiling() const
+{
+    return m_tilingModeDepth;
+}
+
+/*!
+    \qmlproperty real QtQuick3D::Texture::rotationUV
 
     This property rotates the texture around the pivot point. This is defined
     using euler angles and for a positive value rotation is clockwise.
@@ -347,7 +366,7 @@ float QQuick3DTexture::rotationUV() const
 }
 
 /*!
-    \qmlproperty float QtQuick3D::Texture::positionU
+    \qmlproperty real QtQuick3D::Texture::positionU
 
     This property offsets the U coordinate mapping from left to right.
 
@@ -368,7 +387,7 @@ float QQuick3DTexture::positionU() const
 }
 
 /*!
-    \qmlproperty float QtQuick3D::Texture::positionV
+    \qmlproperty real QtQuick3D::Texture::positionV
 
     This property offsets the V coordinate mapping from bottom to top.
 
@@ -393,7 +412,7 @@ float QQuick3DTexture::positionV() const
 }
 
 /*!
-    \qmlproperty float QtQuick3D::Texture::pivotU
+    \qmlproperty real QtQuick3D::Texture::pivotU
 
     This property sets the pivot U position which is used when applying a
     \l{QtQuick3D::Texture::rotationUV}{rotationUV}.
@@ -415,7 +434,7 @@ float QQuick3DTexture::pivotU() const
 }
 
 /*!
-    \qmlproperty float QtQuick3D::Texture::pivotV
+    \qmlproperty real QtQuick3D::Texture::pivotV
 
     This property sets the pivot V position which is used when applying a
     \l{QtQuick3D::Texture::rotationUV}{rotationUV}.
@@ -495,8 +514,8 @@ int QQuick3DTexture::indexUV() const
 /*!
     \qmlproperty enumeration QtQuick3D::Texture::magFilter
 
-    This property determines how the texture is sampled when a texel covers
-    more than one pixel.
+    This property determines how the texture is sampled when it is "magnified",
+    i.e. a texel covers \e more than one pixel in screen space.
 
     The default value is \c{Texture.Linear}.
 
@@ -515,8 +534,8 @@ QQuick3DTexture::Filter QQuick3DTexture::magFilter() const
 /*!
     \qmlproperty enumeration QtQuick3D::Texture::minFilter
 
-    This property determines how the texture is sampled when a texel covers
-    more than one pixel.
+    This property determines how the texture is sampled when it is "minimized",
+    i.e. a texel covers \e less than one pixel in screen space.
 
     The default value is \c{Texture.Linear}.
 
@@ -633,6 +652,25 @@ bool QQuick3DTexture::autoOrientation() const
     return m_autoOrientation;
 }
 
+/*!
+    \qmlproperty RenderExtension QtQuick3D::Texture::textureProvider
+
+    This property holds the RenderExtension that will provide the \l QRhiTexture
+    that will be used by this item.
+
+    \note The texture created by RenderExtension needs to be made available by
+    \l{QSSGRenderExtensionHelpers::registerRenderResult}{registering} it with the engine.
+
+    \since 6.7
+
+    \sa RenderExtension, QSSGRenderExtensionHelpers
+*/
+
+QQuick3DRenderExtension *QQuick3DTexture::textureProvider() const
+{
+    return m_renderExtension;
+}
+
 void QQuick3DTexture::setSource(const QUrl &source)
 {
     if (m_source == source)
@@ -778,6 +816,15 @@ void QQuick3DTexture::setVerticalTiling(QQuick3DTexture::TilingMode tilingModeVe
     update();
 }
 
+void QQuick3DTexture::setDepthTiling(QQuick3DTexture::TilingMode tilingModeDepth)
+{
+    if (m_tilingModeDepth == tilingModeDepth)
+        return;
+    m_tilingModeDepth = tilingModeDepth;
+    emit depthTilingChanged();
+    update();
+}
+
 void QQuick3DTexture::setRotationUV(float rotationUV)
 {
     if (qFuzzyCompare(m_rotationUV, rotationUV))
@@ -878,9 +925,7 @@ void QQuick3DTexture::setTextureData(QQuick3DTextureData *textureData)
         return;
 
     // Make sure to disconnect if the geometry gets deleted out from under us
-    QQuick3DObjectPrivate::updatePropertyListener(textureData, m_textureData, QQuick3DObjectPrivate::get(this)->sceneManager, QByteArrayLiteral("textureData"), m_connections, [this](QQuick3DObject *n) {
-        setTextureData(qobject_cast<QQuick3DTextureData *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DTexture::setTextureData, textureData, m_textureData);
 
     if (m_textureData)
         QObject::disconnect(m_textureDataConnection);
@@ -888,7 +933,7 @@ void QQuick3DTexture::setTextureData(QQuick3DTextureData *textureData)
 
     if (m_textureData) {
         m_textureDataConnection
-                = QObject::connect(m_textureData, &QQuick3DTextureData::textureDataNodeDirty, [this]() {
+                = QObject::connect(m_textureData, &QQuick3DTextureData::textureDataNodeDirty, this, [this]() {
             markDirty(DirtyFlag::TextureDataDirty);
         });
     }
@@ -975,10 +1020,10 @@ bool QQuick3DTexture::effectiveFlipV(const QSSGRenderImage &imageNode) const
     if (m_sourceItem)
         return !m_flipV;
 
-    // With textureData we assume the application knows what it is doing,
+    // With textureData and renderExtension we assume the application knows what it is doing,
     // because there the application is controlling the content itself.
 
-    if (m_textureData)
+    if (m_textureData || m_renderExtension)
         return m_flipV;
 
     // Compressed textures (or any texture that is coming from the associated
@@ -1000,13 +1045,35 @@ bool QQuick3DTexture::effectiveFlipV(const QSSGRenderImage &imageNode) const
     return m_flipV;
 }
 
+static QSSGRenderPath resolveImagePath(const QUrl &url, const QQmlContext *context)
+{
+    if (context && url.isRelative()) {
+        QString path = url.path();
+        QChar separator = QChar::fromLatin1(';');
+        if (path.contains(separator)) {
+            QString resolvedPath;
+            const QStringList paths = path.split(separator);
+            bool first = true;
+            for (auto &s : paths) {
+                auto mapped =  QQmlFile::urlToLocalFileOrQrc(context->resolvedUrl(s));
+                if (!first)
+                    resolvedPath.append(separator);
+                resolvedPath.append(mapped);
+                first = false;
+            }
+            return QSSGRenderPath(resolvedPath);
+        }
+    }
+    return QSSGRenderPath(QQmlFile::urlToLocalFileOrQrc(context ? context->resolvedUrl(url) : url));
+}
+
 QSSGRenderGraphObject *QQuick3DTexture::updateSpatialNode(QSSGRenderGraphObject *node)
 {
     if (!node) {
         markAllDirty();
         node = new QSSGRenderImage(QQuick3DObjectPrivate::get(this)->type);
     }
-
+    QQuick3DObject::updateSpatialNode(node);
     auto imageNode = static_cast<QSSGRenderImage *>(node);
 
     if (m_dirtyFlags.testFlag(DirtyFlag::TransformDirty)) {
@@ -1028,7 +1095,7 @@ QSSGRenderGraphObject *QQuick3DTexture::updateSpatialNode(QSSGRenderGraphObject
         m_dirtyFlags.setFlag(DirtyFlag::FlipVDirty, true);
         if (!m_source.isEmpty()) {
             const QQmlContext *context = qmlContext(this);
-            imageNode->m_imagePath = QSSGRenderPath(QQmlFile::urlToLocalFileOrQrc(context ? context->resolvedUrl(m_source) : m_source));
+            imageNode->m_imagePath = resolveImagePath(m_source, context);
         } else {
             imageNode->m_imagePath = QSSGRenderPath();
         }
@@ -1044,6 +1111,8 @@ QSSGRenderGraphObject *QQuick3DTexture::updateSpatialNode(QSSGRenderGraphObject
                                   QSSGRenderTextureCoordOp(m_tilingModeHorizontal));
     nodeChanged |= qUpdateIfNeeded(imageNode->m_verticalTilingMode,
                                   QSSGRenderTextureCoordOp(m_tilingModeVertical));
+    nodeChanged |= qUpdateIfNeeded(imageNode->m_depthTilingMode,
+                                  QSSGRenderTextureCoordOp(m_tilingModeDepth));
 
     if (m_dirtyFlags.testFlag(DirtyFlag::SamplerDirty)) {
         m_dirtyFlags.setFlag(DirtyFlag::SamplerDirty, false);
@@ -1067,6 +1136,23 @@ QSSGRenderGraphObject *QQuick3DTexture::updateSpatialNode(QSSGRenderGraphObject
         nodeChanged = true;
     }
 
+    if (m_dirtyFlags.testFlag(DirtyFlag::ExtensionDirty)) {
+        bool extDirty = false;
+        if (m_renderExtension) {
+            auto *sn = QQuick3DObjectPrivate::get(m_renderExtension)->spatialNode;
+            // NOTE: We don't clear if we haven't gotten the spatial node yet, as
+            // we'll be called once _again_ when the extensions have been processed.
+            extDirty = (sn == nullptr);
+            if (sn && QSSG_GUARD(sn->type == QSSGRenderGraphObject::Type::RenderExtension))
+                imageNode->m_extensionsSource = static_cast<QSSGRenderExtension *>(sn);
+        }
+
+        m_dirtyFlags.setFlag(DirtyFlag::ExtensionDirty, extDirty);
+        m_dirtyFlags.setFlag(DirtyFlag::FlipVDirty, true);
+
+        nodeChanged = true;
+    }
+
     if (m_dirtyFlags.testFlag(DirtyFlag::SourceItemDirty)) {
         m_dirtyFlags.setFlag(DirtyFlag::SourceItemDirty, false);
         m_dirtyFlags.setFlag(DirtyFlag::FlipVDirty, true);
@@ -1085,9 +1171,14 @@ QSSGRenderGraphObject *QQuick3DTexture::updateSpatialNode(QSSGRenderGraphObject
                     qWarning("Unable to get window, this will probably not work");
             }
 
+            // Workaround: Due to limitation in how the texture provider works in View3D we can't use it in the
+            // special case when a non-visible View3D is used as a sourceItem, at least not in its current incarnation.
+            // There's also a drawback here, since we now have an extra indirection by rendering to the layer texture first..
+            const bool isHiddenView3D = !m_sourceItem->isVisible() && (qobject_cast<QQuick3DViewport *>(m_sourceItem) != nullptr);
+
             // This assumes that the QSGTextureProvider returned never changes,
             // which is hopefully the case for both Image and Item layers.
-            if (QSGTextureProvider *provider = m_sourceItem->textureProvider()) {
+            if (QSGTextureProvider *provider = m_sourceItem->textureProvider(); provider != nullptr && !isHiddenView3D) {
                 imageNode->m_qsgTexture = provider->texture();
 
                 disconnect(m_textureProviderConnection);
@@ -1383,4 +1474,22 @@ void QQuick3DTexture::markAllDirty()
     QQuick3DObject::markAllDirty();
 }
 
+void QQuick3DTexture::setTextureProvider(QQuick3DRenderExtension *textureProvider)
+{
+    if (m_renderExtension == textureProvider)
+        return;
+
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DTexture::setTextureProvider, textureProvider, m_renderExtension);
+
+    m_renderExtension = textureProvider;
+
+    m_dirtyFlags.setFlag(DirtyFlag::SourceDirty);
+    m_dirtyFlags.setFlag(DirtyFlag::SourceItemDirty);
+    m_dirtyFlags.setFlag(DirtyFlag::TextureDataDirty);
+    m_dirtyFlags.setFlag(DirtyFlag::ExtensionDirty);
+
+    emit textureProviderChanged();
+    update();
+}
+
 QT_END_NAMESPACE
