@@ -29,7 +29,10 @@ struct Q_QUICK3DRUNTIMERENDER_EXPORT QSSGRenderMorphTarget : public QSSGRenderGr
         Position   = 0x01,
         Normal     = 0x02,
         Tangent    = 0x04,
-        Binormal   = 0x08
+        Binormal   = 0x08,
+        TexCoord0  = 0x10,
+        TexCoord1  = 0x20,
+        Color      = 0x40
     };
     Q_DECLARE_FLAGS(InputAttributes, InputAttribute);
 
