@@ -1,5 +1,5 @@
 // Copyright (C) 2020 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "parser.h"
 
@@ -399,10 +399,8 @@ static void cloneQmlList(const QObject &so, QMetaProperty &sp, QObject &to, QMet
         auto tl = tv.value<QQmlListProperty<T>>();
         auto sl = sv.value<QQmlListProperty<T>>();
         const auto count = sl.count(&sl);
-        for (int i = 0; count != i; ++i) {
-            const auto &item = sl.at(&sl, i);
-            tl.append(&tl, item);
-        }
+        for (int i = 0; count != i; ++i)
+            tl.append(&tl, sl.at(&sl, i));
     }
 }
 
@@ -1199,7 +1197,7 @@ static bool interceptObjectDef(const QQmlJS::AST::UiObjectDefinition &def, Conte
     // If this is a new component register it
     if (ctx.type == Context::Type::Component && ctx.property.target == nullptr && type != -1) {
         const auto &fileName = ctx.currentFileInfo.fileName();
-        componentName = fileName.left(fileName.length() - 4);
+        componentName = fileName.left(fileName.size() - 4);
         doRegisterComponent = !componentName.isEmpty();
     }
 
